'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _bemNaming = require('bem-naming');

var _bemNaming2 = _interopRequireDefault(_bemNaming);

var _utils = require('./utils');

exports['default'] = function (options) {
    // defaults options:
    // {
    //     elem: '_',
    //     mod: '__'
    //     wordPattern: '[a-z0-9]+(?:-[a-z0-9]+)*'
    // }
    var bemNaming = (0, _bemNaming2['default'])(options);

    return function (chai) {
        var Assertion = chai.Assertion;
        var _chai$assert = chai.assert;
        var typeOf = _chai$assert.typeOf;
        var isTrue = _chai$assert.isTrue;

        Assertion.addMethod('block', function (blockName) {
            var entity = this._obj;

            // check input
            isTrue((0, _utils.isClassName)(entity) || (0, _utils.isDOMElement)(entity), entity + ' is neither a DOM element nor a className');
            typeOf(blockName, 'string');

            // retrieve and prepare className
            var className = (0, _utils.prepareClassName)(entity);

            // check if there is at least one valid block
            isTrue(className.some(function (item) {
                return bemNaming.isBlock(item);
            }), 'there is no valid block in ' + entity);

            // check block
            this.assert(className.indexOf(blockName) !== -1, 'expected #{this} to be #{exp} block but got #{act}', 'expected #{this} to not be #{exp} block but got #{act}', blockName, entity);
        });

        Assertion.addProperty('validBlock', function () {
            var entity = this._obj;

            // check input
            isTrue((0, _utils.isClassName)(entity) || (0, _utils.isDOMElement)(entity), entity + ' is neither a DOM element nor a className');

            // retrieve and prepare className
            var className = (0, _utils.prepareClassName)(entity);

            // check block
            this.assert(className.some(function (item) {
                return bemNaming.isBlock(item);
            }), 'expected #{this} to be a block but got #{act}', 'expected #{this} to not be a block but got #{act}', entity, entity);
        });

        Assertion.addMethod('elem', function (bemjson) {
            var entity = this._obj;

            // check input
            isTrue((0, _utils.isClassName)(entity) || (0, _utils.isDOMElement)(entity), entity + ' is neither a DOM element nor a className');
            typeOf(bemjson, 'object');

            // retrieve and prepare className
            var className = (0, _utils.prepareClassName)(entity);

            // check element
            this.assert(className.some(function (item) {
                // ignore if not element
                if (!bemNaming.isElem(item)) {
                    return false;
                }

                var parsedItem = bemNaming.parse(item);

                return parsedItem.block === bemjson.block && parsedItem.elem === bemjson.elem;
            }), 'expected #{this} to be an elem #{exp} but got #{act}', 'expected #{this} to not be an elem of #{exp} block, but got #{act}', bemjson, entity);
        });

        Assertion.addProperty('validElem', function () {
            var entity = this._obj;

            // check input
            isTrue((0, _utils.isClassName)(entity) || (0, _utils.isDOMElement)(entity), entity + ' is neither a DOM element nor a className');

            // retrieve and prepare className
            var className = (0, _utils.prepareClassName)(entity);

            // check block
            this.assert(className.some(function (item) {
                return bemNaming.isElem(item);
            }), 'expected #{this} to be an element but got #{act}', 'expected #{this} to not be an element but got #{act}', entity, entity);
        });

        Assertion.addMethod('mods', function (bemjson) {
            var entity = this._obj;

            // check input
            isTrue((0, _utils.isClassName)(entity) || (0, _utils.isDOMElement)(entity), entity + ' is neither a DOM element nor a className');
            typeOf(bemjson, 'object');

            // retrieve and prepare className
            var className = (0, _utils.prepareClassName)(entity);

            var mods = Object.keys(bemjson);
            var modWord = 'modificator' + (mods.length === 1 ? '' : 's');

            // check mods
            this.assert(mods.every(function (modName) {
                // parsing to string to handle boolean and numeric mod values
                var modVal = String(bemjson[modName]);

                return className.some(function (item) {
                    // ignore if no mods
                    if (!bemNaming.isBlockMod(item) && !bemNaming.isElemMod(item)) {
                        return false;
                    }

                    var parsedItem = bemNaming.parse(item);

                    return parsedItem.modName === modName && String(parsedItem.modVal) === modVal;
                });
            }), 'expected #{this} to have #{exp} ' + modWord, 'expected #{this} to not have #{exp} ' + modWord, bemjson, entity);
        });
    };
};

module.exports = exports['default'];